@font-family: Verdana, Arial, Helvetica, Sans-Serif;

/* RB Colors */

@default-text-color: #000000;

@light-yellow: #FFF47C;
@dark-yellow: #A89C00;
@light-grey: #EAEAEA;
@grey: #D0D0D0;
@dark-grey: #5C5C5C;
@light-blue: #D0E6FF;
@dark-blue: #3465A4;

@link-color: #0000C0;

@required-flag-color: darkred;

@admin_font_size: 11px;

@btn-primary-color1: #7E9BC6;
@btn-primary-color2: darken(@btn-primary-color1, 20%);

@page-container-padding: 1.5em;

@headerbar-logo-padding-horiz: 8px;
@headerbar-logo-width: 60px;
@headerbar-padding-left: @headerbar-logo-width +
                         2 * @headerbar-logo-padding-horiz;
@headerbar-padding: 5px;
@headerbar-menu-min-width: 9em;

@box-border-color: #888A85;
@box-border-radius: 6px;
@box-inner-border-radius: @box-border-radius - 1px;
@box-title-bg: @light-blue;
@box-shadow: 1px 1px 2px rgba(0, 0, 0, .15);
@box-padding: 10px;
@box-bg: #FFFFFF;

@edit-icon-margin: 4px;

@draft-bg-color: #CDFF9C;
@draft-border-color: darken(@draft-bg-color, 50%);

@review-request-bg: #FEFADF;
@review-request-bg-gradient-start: lighten(@review-request-bg, 4%);
@review-request-bg-gradient-end: @review-request-bg;
@review-request-border-color: #9E9A7F;

@review-request-action-bg: #E5D7A8;
@review-request-action-border-color: darken(@review-request-action-bg, 35%);
@review-request-action-active-bg: darken(@review-request-action-bg, 20%);
@review-request-action-hover-bg: darken(@review-request-action-bg, 12%);

@review-request-action-primary-bg: @review-request-action-bg;
@review-request-action-primary-active-bg: @review-request-action-active-bg;
@review-request-action-primary-hover-bg: @review-request-action-hover-bg;

@review-request-action-menu-border-color: @review-request-action-border-color;
@review-request-action-menu-bg: @review-request-action-active-bg;
@review-request-action-menu-item-hover-bg: @review-request-action-hover-bg;
@review-request-action-padding-vert: 0.6em;
@review-request-action-padding-horiz: 1.2em;

@review-request-action-mobile-padding: 0.3em;

@review-request-label-color: #575012;
@review-request-link-color: #0000EE;

@review-request-warning-bg: #F5A9A9;
@review-request-warning-border-color: #FF0000;

// Reviews
@reviews-padding-horiz: 0.5em;
@review-box-border-color: #999999;

// Review UIs
@review-ui-header-bg: @review-request-bg;

@admin-sidebar-width: 22em;

@admin-msg-banner-bg: #FFC;
@admin-msg-banner-border-color: darken(@admin-msg-banner-bg, 50%);

@issue-table-border-color: #999;
@issue-table-cell-padding: 7px;
@issue-table-header-bg: #E9E9E9;
@issue-table-header-hover-bg: darken(@issue-table-header-bg, 5%);
@issue-table-row-border-color: #CCC;
@issue-table-tab-border-color: @issue-table-border-color;
@issue-table-tab-bg: @issue-table-header-bg;
@issue-table-tab-hover-bg: lighten(@issue-table-header-bg, 5%);
@issue-table-tab-hover-border-color: lighten(@issue-table-tab-border-color, 5%);

@issue-opened-bg: #fff4B0;
@issue-opened-border-color: darken(@issue-opened-bg, 40%);
@issue-opened-hover-bg: desaturate(darken(@issue-opened-bg, 10%), 20%);
@issue-opened-link-color: darken(@issue-opened-bg, 60%);
@issue-discarded-bg: #ECECEC;
@issue-discarded-border-color: darken(@issue-discarded-bg, 20%);
@issue-discarded-hover-bg: darken(@issue-discarded-bg, 5%);
@issue-discarded-link-color: darken(@issue-discarded-bg, 60%);
@issue-resolved-bg: #FAFAFA;
@issue-resolved-border-color: darken(@issue-resolved-bg, 10%);
@issue-resolved-hover-bg: darken(@issue-resolved-bg, 5%);
@issue-resolved-link-color: darken(@issue-resolved-bg, 60%);

/*
 * At least on Chrome, using points instead of pixels does not reliably
 * result in identical sizings when there are elements positioned inside
 * a <pre>. Pixels fixes this.
 */
@textarea-font-size: 13px;
@textarea-font-family: monospace;
@textarea-line-height: (@textarea-font-size + 2px);
@textarea-border-color: #b8b5a0;
@textarea-border: 1px #b8b5a0 solid;
@textarea-editor-background: white;
@textarea-editor-margin: 10px 0 0 0;
@textarea-editor-padding: 10px;

@z-index-base: 1;
@z-index-deco: 20;
@z-index-event-overlay: 30;
@z-index-subevent-overlay: @z-index-event-overlay + 1;
@z-index-filter-overlay: 40;
@z-index-mobile-headerbar: 50;
@z-index-logo: @z-index-mobile-headerbar;
@z-index-banner: 90;
@z-index-diff-collapse-btn: 100;
@z-index-menu: 110;
@z-index-dialog: 120;
@z-index-page-overlay: 130;
@z-index-spinner: 140;
@z-index-nav-menu-mask: 199;
@z-index-nav-menu: 200;

// Icon dimensions
@rb-icon-edit-width: 20px;

// Comment flags
@comment-flag-color: #a1c0e3;
@comment-flag-hover-color: #a5caf3;
@comment-flag-border-color: #204a87;
@comment-flag-border-radius: 4px;

@comment-flag-draft-color: #aceb6f;
@comment-flag-draft-hover-color: #b6f37c;
@comment-flag-draft-border-color: #3e7b01;

@comment-flag-ghost-color: #eeeeec;
@comment-flag-ghost-border-color: #888a85;

// Comment dialog
@comment-dlg-bg: #ACEB6F;
@comment-dlg-border-color: darken(@comment-dlg-bg, 40%);
@comment-dlg-padding: 6px;
@comment-dlg-other-comments-bg: #A1C0E3;

/* Diff styles */
@diff-file-color: #F0F0F0;
@diff-border-color: #A0A0A0;
@diff-file-border-color: darken(@diff-file-color, 15%);
@diff-line-border-color: #D0D0D0;
@diff-error-color: #FFF6A6;
@diff-error-border-color: darken(#FFF6A6, 35%);

// Diff headers
@diff-header-color: #e4d9cb;

// Moved line information
@diff-moved-from-bg-color: #b0d0ff;
@diff-moved-to-bg-color: darken(@diff-moved-from-bg-color, 3%);

// Equals lines
@diff-linenum-color: #777777;
@diff-linenum-background-color: #F0F0F0;
@diff-selected-color: #E2E2E2;

// Delete lines
@diff-delete-color: #ffe0e5;
@diff-delete-border-color: desaturate(darken(@diff-delete-color, 7%), 55%);
@diff-delete-linenum-color: desaturate(darken(@diff-delete-color, 5%), 35%);
@diff-delete-linenum-border-color:
  desaturate(darken(@diff-delete-linenum-color, 7%), 55%);
@diff-delete-selected-color: darken(@diff-delete-linenum-color, 4%);
@diff-delete-dot-color: desaturate(darken(@diff-delete-color, 30%), 35%);

// Insert lines
@diff-insert-color: #dfffd7;
@diff-insert-border-color: desaturate(darken(@diff-insert-color, 7%), 55%);
@diff-insert-linenum-color: desaturate(darken(@diff-insert-color, 7%), 35%);
@diff-insert-linenum-border-color:
  desaturate(darken(@diff-insert-linenum-color, 7%), 35%);
@diff-insert-selected-color: darken(@diff-insert-linenum-color, 5%);
@diff-insert-dot-color: desaturate(darken(@diff-insert-color, 35%), 35%);

// Replace lines
@diff-replace-color: #fdfecc;
@diff-replace-border-color: desaturate(darken(@diff-replace-color, 20%), 30%);
@diff-replace-linenum-color: desaturate(darken(@diff-replace-color, 7%), 25%);
@diff-replace-linenum-border-color:
  desaturate(darken(@diff-replace-linenum-color, 10%), 30%);
@diff-replace-selected-color: darken(@diff-replace-linenum-color, 10%);
@diff-replace-dot-color: darken(@diff-replace-color, 50%);

// Diff line counts
@diff-insert-line-count-color: darkgreen;
@diff-delete-line-count-color: darkred;

// Diff highlighted line
@diff-highlight-color: #F3CBFF;

// Revisions selector
@revisions-border-color: #999999;
@revisions-hover-color: #E7E0A2;
@revisions-hover-border-color: #003366;
@revisions-text-color: black;

// Pagination
@paginate-border-color: #DDDDDD;
@paginate-bg-color: #417690;
@paginate-text-color: #f4f379;

// Binary Files
@inline-actions-bg: darken(@diff-file-color, 3%);
@inline-actions-hover-bg: darken(@inline-actions-bg, 5%);

// Datagrids
@datagrid-bg: @box-bg;
@datagrid-border-color: @box-border-color;
@datagrid-filter-menu-bg: #FFFFE0;
@datagrid-filter-menu-border-color: darken(@datagrid-filter-menu-bg, 65%);
@datagrid-paginator-bg: #EEE;
@datagrid-paginator-border-color: #BBB;
@datagrid-actions-bg: @datagrid-paginator-bg;
@datagrid-actions-offset-left: 1em;
@datagrid-header-bg: @box-title-bg;
@datagrid-row-even-bg: #FBFBFB;
@datagrid-row-odd-bg: #FFFFFF;
@datagrid-row-border-color: #F0F0F0;

// Page Sidebar
@page-sidebar-bg: #E9E9E9;
@page-sidebar-count-color: #A02222;
@page-sidebar-count-zero-color: #BBBBBB;
@page-sidebar-horiz-padding: 0.8em;
@page-sidebar-vert-padding: 0.4em;
@page-sidebar-item-vert-padding: 6px;
@page-sidebar-section-vert-padding: 8px;
@page-sidebar-active-bg: #FFFFFF;
@page-sidebar-active-border-color: @datagrid-border-color;
@page-sidebar-count-vert-padding: 0px;
@page-sidebar-count-horiz-padding: 8px;
@page-sidebar-hover-bg: lighten(@page-sidebar-active-bg, 30%);
@page-sidebar-hover-border-color: lighten(@page-sidebar-active-border-color, 30%);
@page-sidebar-width: 19em;

// Review request labels in datagrids.
@summary-label-draft-bg: #AAE29B;
@summary-label-draft-border-color: darken(@summary-label-draft-bg, 28%);
@summary-label-text-color: #005500;
@summary-label-submitted-bg: #FFFFD0;
@summary-label-submitted-border-color: darken(@summary-label-submitted-bg, 55%);
@summary-label-submitted-text-color: #777700;
@summary-label-discarded-bg: #DDDDDD;
@summary-label-discarded-border-color: darken(@summary-label-discarded-bg, 25%);
@summary-label-discarded-text-color: #555555;
@summary-label-archived-bg: #FAFAFA;
@summary-label-archived-border-color: darken(@summary-label-archived-bg, 25%);
@summary-label-archived-text-color: #555555;
@summary-label-muted-bg: #DDDDDD;
@summary-label-muted-border-color: darken(@summary-label-muted-bg, 25%);
@summary-label-muted-text-color: #555555;

// New review request page
@new-review-request-bg: #FFFFFF;
@new-review-request-commit-icon-color: #888;
@new-review-request-commit-info-color: #666;
@new-review-request-jump-to-commit-color: #71A6E2;
@new-review-request-summary-posted-color: #777;

// Mobile
@mobile-back-forward-link-color: #444;
@mobile-menu-animate-duration: 0.3s;
@mobile-menu-animate-timing-func: ease-out;
@mobile-menu-bg-color: #383A46;
@mobile-menu-text-color: #EEE;
@mobile-menu-border-color: darken(@mobile-menu-bg-color, 20%);
@mobile-menu-header-bg-color: darken(@mobile-menu-bg-color, 5%);
@mobile-menu-header-border-color: darken(@mobile-menu-header-bg-color, 5%);
@mobile-menu-footer-height: 40px;
@mobile-menu-header-height: 32px;
@mobile-navbar-width: 300px;
@mobile-medium-width-720: 720px;

// Trophy banner
@trophy-banner-bg-color: #fdf2a0;


@import (reference) "mixins/gradients.less";
@import (reference) "mixins/style.less";
@import (reference) "mixins/theme.less";
