{% extends "datagrids/listview.html" %}
{% load djblets_utils i18n reviewtags %}

{% block datagrid_titlebox %}
{%  if datagrid.group %}{% star datagrid.group %}{% endif %}
{{block.super}}
{% endblock datagrid_titlebox %}

{% block datagrid_filters %}
<li class="datagrid-showclosed">
{%  if datagrid.show_closed %}
 <a href="{% querystring_with 'show-closed' '0' %}">{% trans "Hide closed" %}</a></li>
{%  else %}
 <a href="{% querystring_with 'show-closed' '1' %}">{% trans "Show closed" %}</a></li>
{%  endif %}
</li>
{% endblock datagrid_filters%}
