.btn,
input[type="button"],
input[type="submit"] {
  cursor: pointer;
  display: inline-block;
  background-color: #e6e6e6;
  background-repeat: no-repeat;
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#ffffff), color-stop(25%, #ffffff), to(#e6e6e6));
  background-image: -webkit-linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  background-image: -moz-linear-gradient(top, #ffffff, #ffffff 25%, #e6e6e6);
  background-image: -ms-linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  background-image: -o-linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  background-image: linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffffff', endColorstr='#e6e6e6', GradientType=0);
  padding: 3px 8px;
  text-shadow: 0 1px 1px rgba(255, 255, 255, 0.75);
  color: #333;
  font-size: 13px;
  line-height: normal;
  border: 1px solid #ccc;
  border-bottom-color: #bbb;
  -moz-border-radius: 4px;
  -webkit-border-radius: 4px;
  border-radius: 4px;
  -moz-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2) 0 1px 2px rgba(0, 0, 0, 0.05);
  -webkit-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2) 0 1px 2px rgba(0, 0, 0, 0.05);
  box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2) 0 1px 2px rgba(0, 0, 0, 0.05);
  -webkit-transition: 0.1s linear all;
  -moz-transition: 0.1s linear all;
  transition: 0.1s linear all;
}
.btn[disabled],
input[type="button"][disabled],
input[type="submit"][disabled] {
  cursor: default;
  background-image: none;
  filter: progid:DXImageTransform.Microsoft.gradient(enabled = false);
  opacity: 0.65;
  filter: alpha(opacity=65);
  -moz-opacity: 0.65;
  -moz-box-shadow: none;
  -webkit-box-shadow: none;
  box-shadow: none;
}
.btn:active,
input[type="button"]:active,
input[type="submit"]:active {
  -moz-box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.25) 0 1px 2px rgba(0, 0, 0, 0.05);
  -webkit-box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.25) 0 1px 2px rgba(0, 0, 0, 0.05);
  box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.25) 0 1px 2px rgba(0, 0, 0, 0.05);
}
.btn:hover,
input[type="button"]:hover,
input[type="submit"]:hover {
  background-position: 0 -15px;
  color: #333;
  text-decoration: none;
}
.btn.danger,
input[type="button"].danger,
input[type="submit"].danger {
  color: white !important;
  background-color: #c43c35;
  background-repeat: repeat-x;
  background-image: -khtml-gradient(linear, left top, left bottom, from(#ee5f5b), to(#c43c35));
  background-image: -moz-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: -ms-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #ee5f5b), color-stop(100%, #c43c35));
  background-image: -webkit-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: -o-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: linear-gradient(top, #ee5f5b, #c43c35);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ee5f5b', endColorstr='#c43c35', GradientType=0);
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  border-color: #c43c35 #c43c35 #882a25;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
}
.btn.disabled,
input[type="button"].disabled,
input[type="submit"].disabled {
  cursor: default;
  background-image: none;
  filter: progid:DXImageTransform.Microsoft.gradient(enabled = false);
  opacity: 0.65;
  filter: alpha(opacity=65);
  -moz-opacity: 0.65;
  -moz-box-shadow: none;
  -webkit-box-shadow: none;
  box-shadow: none;
}
.btn.primary,
input[type="button"].primary,
input[type="submit"].primary,
.btn.default,
input[type="button"].default,
input[type="submit"].default {
  color: white;
  text-shadow: none;
  background-color: #44679a;
  background-repeat: repeat-x;
  background-image: -khtml-gradient(linear, left top, left bottom, from(#7E9BC6), to(#44679a));
  background-image: -moz-linear-gradient(top, #7E9BC6, #44679a);
  background-image: -ms-linear-gradient(top, #7E9BC6, #44679a);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #7E9BC6), color-stop(100%, #44679a));
  background-image: -webkit-linear-gradient(top, #7E9BC6, #44679a);
  background-image: -o-linear-gradient(top, #7E9BC6, #44679a);
  background-image: linear-gradient(top, #7E9BC6, #44679a);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#7E9BC6', endColorstr='#44679a', GradientType=0);
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  border-color: #44679a #44679a #2d4365;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
}
.btn.large,
input[type="button"].large,
input[type="submit"].large {
  font-size: 130%;
  line-height: normal;
  padding: 9px 14px 9px;
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
}
.btn.small,
input[type="button"].small,
input[type="submit"].small {
  padding: 7px 9px 7px;
  font-size: 11px;
}
/*
 * Force Firefox to render our buttons with our chosen font. Otherwise it will
 * be overridden (on macOS at least) with the system font.
 */
input[type="button"],
input[type="submit"] {
  font-family: Verdana, Arial, Helvetica, Sans-Serif;
}
button.btn::-moz-focus-inner,
input[type=submit].btn::-moz-focus-inner {
  padding: 0;
  border: 0;
}
