@import (reference) "../defs.less";


.banner {
  background: @draft-bg-color;
  border: 1px @draft-border-color solid;
  margin-bottom: 10px;
  padding: 8px 10px;
  z-index: @z-index-banner;

  &>h1, &>p {
    display: inline;
    margin-right: 0.2em;
    padding: 0;
  }

  .on-mobile-medium-screen-720({
    p {
      display: block;
      margin: 1em 0;
    }
  });

  .banner-actions {
    margin-left: 1em;

    .on-mobile-medium-screen-720({
      display: block;
      margin-left: 0;
      margin-top: 1em;
    });
  }
}
