{% extends "accounts/base.html" %}
{% load djblets_deco i18n %}

{% block title %}{% trans "Password Reset Complete" %}{% endblock %}

{% block auth_content %}
<div class="auth-header">
 <h1>{% trans "Password Reset Complete" %}</h1>
</div>

<p>
{%  url 'login' as login_url %}
{%  blocktrans %}
 Your password has been set. You may go ahead and
 <a href="{{login_url}}">log in</a> now.
{%  endblocktrans %}
</p>
{% endblock auth_content %}
