{% load staticfiles %}
<script type="text/javascript" src="{% static "lib/js/jquery-1.11.1.min.js" %}"></script>
<script type="text/javascript" src="{% static "lib/js/jquery-migrate-1.2.1.min.js" %}"></script>
