from __future__ import unicode_literals


FAKE_SCMTOOLS = {
    'rbpowerpack.scmtools.tfs.TFSTool': 'Team Foundation Server',
    'rbpowerpack.scmtools.tfs_git.TFSGitTool': 'Team Foundation Server (git)',
}
