<div class="bug-infobox-content">
 <div class="bug-infobox-text">
  <h2 class="bug_header">{{bug_id}}: {{bug_summary}}</h2>
  <p class="bug_status">Status: {{bug_status}}</p>
  <p class="bug_description">{{bug_description}}</p>
 </div>
</div>
