{% load djblets_deco %}

{% if trophies %}
{%  box 'yay' %}
{%   for trophy in trophies %}
<div>
 <img src="{{trophy.image_url}}" width="{{trophy.image_width}}" height="{{trophy.image_height}}" />
 <h1>{{trophy.text}}</h1>
</div>
{%   endfor %}
{%  endbox 'yay' %}
{% endif %}
